module Owllink
	
	# extends the given xml stream by
	#
	# <owl:ClassAssertion>
	#   <owl:Class abbreviatedIRI="<class_iri>">
	#   <owl:NamedIndividual abbreviatedIRI="<individual_iri>"
	# </owl:ClassAssertion>
	def Owllink.add_class_assertion(xml,class_iri,individual_iri)
		xml.owl(:ClassAssertion) {
			xml.owl(:Class, :abbreviatedIRI => :"#{class_iri}")
			xml.owl(:NamedIndividual, :abbreviatedIRI => :"#{individual_iri}")
		}
    end
	
	# extends the given xml stream by
	#
	# <owl:ObjectPropertyAssertion>
	#   <owl:ObjectProperty abbreviatedIRI="<objectproperty_iri>"/>
	#   <owl:NamedIndividual abbreviatedIRI="<source_individual_iri>"/>
	#   <owl:NamedIndividual abbreviatedIRI="<target_individual_iri>"/>
	# </owl:ObjectPropertyAssertion>
	def Owllink.add_objectproperty_assertion(xml,objectproperty_iri,source_individual_iri,target_individual_iri)
		xml.owl(:ObjectPropertyAssertion) {
			xml.owl(:ObjectProperty, :abbreviatedIRI => :"#{objectproperty_iri}")
			xml.owl(:NamedIndividual, :abbreviatedIRI => :"#{source_individual_iri}")
			xml.owl(:NamedIndividual, :abbreviatedIRI => :"#{target_individual_iri}")
		}
    end
	
	# extends the given xml stream by
	# 
	# <owl:ClassAssertion>
	#   <owl:ObjectExactCardinality cardinality="<cardinality>">
	#     <owl:ObjectProperty abbreviatedIRI="<objectproperty_iri>"/>
	#   </owl:ObjectExactCardinality>
	#   <owl:NamedIndividual abbreviatedIRI="<individual_iri>"/>
	# </owl:ClassAssertion>
	def Owllink.close_role(xml,individual_iri,objectproperty_iri,cardinality)
		xml.owl(:ClassAssertion) {
			xml.owl(:ObjectExactCardinality, :cardinality => :"#{cardinality}") {
				xml.owl(:ObjectProperty, :abbreviatedIRI => :"#{objectproperty_iri}")
			}
			xml.owl(:NamedIndividual, :abbreviatedIRI => :"#{individual_iri}")
		}
    end
	
	# wraps the given block within an OWLlink request message and returns the resulting XML as string
	# accepts XML entities, namespace declarations as hash arguments
	# defines the OWLlink namespace as default
	#  
	#  <?xml version="1.0" encoding="UTF-8"?>
	#  <!DOCTYPE RequestMessage [
	#    <!ENTITIY "<entity_key>" "<entity_value">
	#    ..
	#  ]>
	#  <RequestMessage xmlns="http://www.owllink.org/owllink#"
	#     xmlns:<namespace_key>="<namespace_value>"
	#     ..>
	#    <body>
	#  </RequestMessage>
	def Owllink.request_message(options = {})
		o = {:entities => {}, :namespaces => {}}.with_indifferent_access.merge!(options)
		xml = Builder::XmlMarkup.new(:target => request_string = "", :indent => 2) 
		xml.instruct!
		if !o[:entities].empty?
			xml.declare! :DOCTYPE, :RequestMessage do |x|
				o[:entities].each do |abbrev,uri| x.declare! :ENTITY, :"#{abbrev}", uri end
				end
			end
		n = o[:namespaces].each_with_object({}) { |(k,v), h| h[:"\n  xmlns:#{k}"] = v }
		n[:"\n  xmlns"] = "http://www.owllink.org/owllink#"
		xml.RequestMessage(n) do |x| 
			yield(x)
			end
		return request_string
    end
	
	# sends the given OWLlink XML message string via HTTP to the configured reasoner
	def Owllink.post_request(message)
		Rails.logger.info '---'
		Rails.logger.info 'OWLlink Request:'
		Rails.logger.info message
		headers = {"User-Agent" => "Ruby v1.8"}
		url = URI.parse(AppConfig.owllink_reasoner_url)
		owllink = Net::HTTP.new(url.host,url.port)
		resp, data = owllink.post((url.path.empty? ? "/" : url.path),message,headers)   
		Rails.logger.info '-' 
		Rails.logger.info 'OWLlink Reply: ' 
		Rails.logger.info ' Code = ' + resp.code + '/' + resp.message
		resp.each {|key, val| Rails.logger.info ' ' + key + ' = ' + val}
		Rails.logger.info data
		Rails.logger.info '---'
		return data
    end
	
	def Owllink.get_server(session)
	    begin
	      Server.find(session[:server])
	    rescue 
		    server = Server.new
			server.save
			session[:server] = server.id
			server
        end
    end
    
    def Owllink.delete_server(server)
        if server
          if server.initialized?
             kb = server.kb
			 if kb && !kb.empty?
               msg = Owllink.request_message {|xml| 
				 xml.ReleaseKB(:kb => kb)
               }
               Owllink.post_request(msg)
             end
          end
          server.delete
        end
    end
    
    def Owllink.release_server(session)
        begin
          Owllink.delete_server(Server.find(session[:server]))
          session[:server] = nil
	    rescue 
        end
    end
	
	def Owllink.get_server_id(session)
		server = Owllink.get_server session
		server.id
    end
	
	def Owllink.connected?(session)
		server = Owllink.get_server session
		server.connected?
    end
	
	def Owllink.connect(session)
		server = Owllink.get_server session
		server.connected = true
		server.save
    end
	
	def Owllink.set_kb(session,kb)
		server = Owllink.get_server session
		server.kb = kb
		server.save
    end
	
	def Owllink.get_kb(session)
		server = Owllink.get_server session
		server.kb
    end
	
	def Owllink.set_name(session,name)
		server = Owllink.get_server session
		server.name = name
		server.save
    end
	
	def Owllink.get_name(session)
		server = Owllink.get_server session
		server.name
    end
	
	def Owllink.set_owllink_version(session,version)
		server = Owllink.get_server session
		server.owllink_version = version
		server.save
    end
	
	def Owllink.get_owllink_version(session)
		server = Owllink.get_server session
		server.owllink_version
    end
        
    def Owllink.set_abbreviates_setting(session,b)
		server = Owllink.get_server session
		server.abbreviates_setting = b
		server.save
    end
	
	def Owllink.abbreviates_setting?(session)
		server = Owllink.get_server session
		server.abbreviates_setting?
    end
        
    def Owllink.set_abbreviates_iris(session,b)
		server = Owllink.get_server session
		server.abbreviates_iris = b
		server.save
    end
	
	def Owllink.abbreviates_iris?(session)
		server = Owllink.get_server session
		server.abbreviates_iris?
    end
	
	def Owllink.set_supports_retraction(session,b)
		server = Owllink.get_server session
		server.supports_retraction = b
		server.save
    end
	
	def Owllink.supports_retraction?(session)
		server = Owllink.get_server session
		server.supports_retraction?
    end
	
	def Owllink.set_load_ontologies(session,b)
		server = Owllink.get_server session
		server.load_ontologies = b
		server.save
    end
	
	def Owllink.load_ontologies?(session)
		server = Owllink.get_server session
		server.load_ontologies?
    end
	
	def Owllink.set_initialized(session,b)
		server = Owllink.get_server session
		server.initialized = b
		server.save
    end
	
	def Owllink.initialized?(session)
		server = Owllink.get_server session
		return server.initialized?
    end
	
	def Owllink.disconnect(session)
		server = Owllink.get_server session
		server.connected = false
		server.save
    end
	
end